*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.;


/*******************************************************************/
/* Date: January 2005                                              */
/*******************************************************************/

/*********************************************************************************/
/***                                                                           ***/
/***                            MACROE_V31.SAS                                 ***/
/***                                                                           ***/
/*** This program calculates variance estimates using the bootstrap weights    ***/
/*** for different types of estimators.  Using SAS Macros, this program can    ***/
/*** calculate variance estimates for totals, ratios and differences between   ***/
/*** ratios.  It can also calculate variance estimates for the parameters      ***/
/*** of a linear regression or logistic regression. This program can also be   ***/
/*** customized for other types of analyses.                                   ***/
/***                                                                           ***/
/*** This program contains the macros that are necessary to use the 	       ***/
/*** BOOTVARE_V31.SAS program.                                                 ***/
/***                                                                           ***/
/*** This program is automatically called by BOOTVARE_V31.SAS and NO MODIFICA- ***/
/*** TIONS SHOULD BE MADE BY THE USER (except for specific cases mentioned in  ***/
/*** documentation)                                                            ***/
/***                                                                           ***/
/*********************************************************************************/


options ls=120 nonotes;


*************************************************************************************
*** Section 1: DECLARATION OF THE MACRO VARIABLES (CAN BE MODIFIED BY THE USERS)  ***                             ***
*************************************************************************************;

	%let deff=0;        * Activate the design effect calculation (total and ratio);
	%let alpha=0.05;    * Set the confidence level for the computation of the confidence intervals;


*************************************************************************************************
***  Section 2: CREATION OF THE MACRO VARIABLES USED BY THE PROGRAM (SHOULD NOT BE MODIFIED)  ***
*************************************************************************************************;

*  VERIFICATION IF BREAKDOWD VARIABLE(S) IS (ARE) SPECIFIED; 

		%let by=;
		%let cla_tmp="&classes";

		data _NULL_;
			if substr(&cla_tmp,1,1)='.' then do; 
         									call symput ('by' ,'*');
         									call symput ('classes','');
         									call symput ('number',0);
										end;
		run;


* TO OBTAIN THE MARGINALS IN THE OUTPUT (The macro vraiable classes is split into several variables);

		&by  %let cla_tmp="&classes"||' #';

		&by  data _NULL_;
		&by  do i=1 to 10;
		&by  	call symput ('cla'||left(trim(i)),' ');
		&by  end;
		&by  init=1; &by  i=1; &by  fin=1; &by  stop=' ';
		&by  do until (stop='#');
		&by  	do until (substr(&cla_tmp,i,1)='');
		&by  		call symput ('cla'||left(trim(init)),substr(&cla_tmp,fin,i-fin+1));
		&by  		call symput ('number',init);
		&by  		i=i+1;
		&by  	end;
		&by  	fin=i;
		&by  	stop=substr(&cla_tmp,i+1,1);
		&by  	init=init+1;
		&by  end;
		&by  run;


* UNIFORMIASTION OF THE VARIABLE NAMES SPECIFIC TO EACH VERSION OF SAS ;

		data _NULL_;
			if &sysver >= 8 then call symput ('inter' ,'intercept');
			else call symput ('inter' ,'intercep');
		run;


* VARIABLES USED FOR THE INDEX (next section) ;

		%let indx=(id=(&ident blank)); 


OPTIONS notes;



*****************************************************************************
***  Section 3: READING IN THE MAIN FILE AND MERGING TO THE WEIGHTS       ***
*****************************************************************************;

		data Mfile (index=&indx);
		   set &Mfile ;
		   blank=.;
		run;

*** The next step reads the bootstrap weights and standardize the name of the bootatrap weigths varaibles ***;

		OPTIONS nonotes;

		data bsamp (index=&indx);
		  set &bsamp;
		   blank=.;
		         if &bsw.1 ne '' then do; 
		                           call symput ('bsw_frst' ,'&bsw.1');
		                           call symput ('bsw_last' ,'&bsw.&b');
								end;

		         else if &bsw.001 ne '' then do; 
		                           call symput ('bsw_frst' ,'&bsw.001');
		                           call symput ('bsw_last' ,'&bsw.%sysfunc(putn(&b,z3.))');
								end;
		         else if &bsw.0001 ne '' then do; 
		                           call symput ('bsw_frst' ,'&bsw.0001');
		                           call symput ('bsw_last' ,'&bsw.%sysfunc(putn(&b,z4.))');
								end;   
		run;

		OPTIONS notes;


*** Merging the analysis file and the bootstrap weights ***;

	     data bs_data ;
	           merge Mfile (in=in1) bsamp (keep= &ident &fwgt &bsw_frst-&bsw_last);
	           by &ident;
	           if in1;
		       drop blank;
	           rename &bsw_frst-&bsw_last=bsw1-bsw&b; 
	     run;


		&by  proc sort data=bs_data;
		&by     by &classes;
		&by  run;


*** Initialise the result files  ;

		data alltots allrats diffrat allperc bs_reg bs_reglg allchi2 ;
		      set _NULL_;
		run;

		%let result= alltots allrats diffrat allperc bs_reg bs_reglg allchi2;


*********************************************************************************
*** Section 4: DECLARATION OF THE MACROS                                      ***
*********************************************************************************;

*** Initialise some macro variables used in the macros ***;

	%let printtot=0;
	%let printrat=0;
	%let printdif=0;
	%let printperc=0;
	%let printreg=0;
	%let printlog=0;
	%let printchi2=0;

	%let model_reg=0;
	%let model_log=0;
	%let test_chi2=0;
	%let test_num2=0;
	%let flag_rat=0;
	%let flag_dif=0;
	%let flag_perc=0;
	%let flag_reg=0;
	%let flag_log=0;
	%let ci_low=0;
	%let ci_upp=0;
	%let ci=0;


************************************************
****                                        ****
****                 TOTAL                  ****
****                                        ****
************************************************;


%macro total(var);

*** Determine if negative values are present ***;

		%let negative=0;
		data _NULL_;
			set bs_data (keep=&var);
			if &var ne 0 and &var / abs(&var)=-1  then do; call symput('negative',1); end;
		run;

*** If no negative values are present, use the fast method ***;

	%if (&negative=0) %then %do;
		proc means data=bs_data noprint;
		    var &fwgt bsw1-bsw&b;
		    weight &var;
		&by  class &classes;
		    output out=ytot
		         sum=yhat ybs1-ybs&b;
		run;
	%end;

*** If negative values are present, use the long method ***;

	%if (&negative=1) %then %do;
		OPTIONS nonotes;
		proc means data=bs_data noprint;
		    var &var;
		    weight &fwgt;
		&by  class &classes;
		  output out=ytot sum=yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=ytot_bsw sum=ybs&looper;
			run;

			&by proc sort data=ytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot ytot_bsw;
				&by by &classes;
			run;
		%end;
		OPTIONS notes;
	%end;

*** Calculation of the sample size  ***;

		proc means data=Mfile noprint;
		    var &var;
		    where &var>0;
		&by  class &classes;
		    output out=n n=n;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		&by proc sort data=n;
		&by 	by &classes;
		&by run;

		data ytot;
			merge ytot n (drop=_type_ _freq_);
		&by by &classes;
			if (&negative=1) then n=.;   * Set the sample size to missing if negative values are present to avoid wrong interpretations;
		run;

*** Variables for the confidence level ;

		%let level=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('CI' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('CI_low',compress("CI_l&CI"));
	 		call symput ('CI_upp',compress("CI_u&CI"));
		run;


*** Calculation of the results ***;

		data est;
			  set ytot;
			  length var $ 8;
			  length type $ 8;
			  Estimate=yhat;
		      bs_var=&r*((&b-1)*(var(of ybs1-ybs&b)))/&b;
			  bs_sd=sqrt(bs_var);
			  if Estimate ne 0 then bs_cv=abs(round((bs_sd/Estimate)*100,.01));
			  else bs_cv=.;
			  &CI_low=Estimate-&level*bs_sd;
			  &CI_upp=Estimate+&level*bs_sd;
			  var="&var";
			  type="Total";
			  drop ybs1-ybs&b yhat _type_ _freq_;
		run;

*** Calculation of the design effect (if activated at the beginning of the program) ***;

		%if (&deff=1) %then %do;
			proc means data=bs_data noprint;
			    var &fwgt;
			&by  class &classes;
			  output out=deff1 n=nsamp sum=npop;

			proc means data=bs_data noprint vardef=wdf;
			    var &var;
		        weight &fwgt;
			&by  class &classes;
			  output out=deff2 var=yvar;

			&by proc sort data=deff1;
			&by 	by &classes;
			&by run;
			&by proc sort data=deff2;
			&by 	by &classes;
			&by run;

			data est;
			    merge est deff1 deff2;
			    &by 	by &classes;
 			    vsrs=npop*npop*yvar/nsamp; 
			    deff=bs_var/vsrs; 
			    drop npop nsamp yvar vsrs _type_ _freq_; 

			proc datasets library=work;
			    delete deff1 deff2;
			run;
		%end;

*** End of the design effect calculation ***;

*** Creation of the result file ***;

		&by   data est;
		&by      set est;
		&by  			%do k=1 %to 10;
		&by  			cla&k='        ';
		&by  			%end;
		&by  			%do k=1 %to &number; 
		&by  			cla&k=input(compress(&&cla&k),$8.);
		&by  			%end;
		&by   run;

		data alltots;
			set alltots est;
		run;

		%let printtot=1;

		proc datasets library=work;
			delete ytot est n;
		run;


%mend total;



************************************************
****                                        ****
****                 RATIO                  ****
****                                        ****
************************************************;


%macro ratio(var1,var2);

*** Determine if negative values are present ***;

		%let negative=0;
		data _NULL_;
			set bs_data (keep=&var1 &var2 );
			if &var1 ne 0 and &var1 / abs(&var1)=-1  then do; call symput('negative',1); end;
			if &var2 ne 0 and &var2 / abs(&var2)=-1  then do; call symput('negative',1); end;
		run;

*** If no negative values are present, use the fast method ***;

	%if (&negative=0) %then %do;
		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var1;
		&by  class &classes;
		  output out=ytot
		         sum=yhat ybs1-ybs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var2;
		  &by  class &classes;
		  output out=xtot
		  sum=xhat xbs1-xbs&b;
		run;
	%end;


*** If negative values are present, use the long method ***;

	%if (&negative=1) %then %do;
		OPTIONS nonotes;
		proc means data=bs_data noprint;
		  var &var1;
		  weight &fwgt;
		&by  class &classes;
		  output out=ytot sum=yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var1;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=ytot_bsw sum=ybs&looper;
			run;

			&by proc sort data=ytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot ytot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		  var &var2;
		  weight &fwgt;
		&by  class &classes;
		  output out=xtot sum=xhat;
		run;

		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var2;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=xtot_bsw sum=xbs&looper;
			run;

			&by proc sort data=xtot_bsw;
			&by 	by &classes;
			&by	run;
			
			data xtot;	
				merge xtot xtot_bsw;
				&by by &classes;
			run;
		%end;

		OPTIONS notes;
	%end;


*** Calculation of the sample size  ***;

		proc means data=Mfile noprint;
		     var &var1;
		     where &var1>0;
		&by  class &classes;
		     output out=n n=n1;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;

		&by proc sort data=n;
		&by 	by &classes;
		&by run;

		data ytot;
		 	 merge ytot n (drop=_type_ _freq_);
		&by by  &classes;
			if (&negative=1) then n1=.;   * Set the sample size to missing if negative values are present to avoid wrong interpretations;
		run;


*** Variables for the confidence level ;

		%let level=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('CI' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('CI_low',compress("CI_l&CI"));
	 		call symput ('CI_upp',compress("CI_u&CI"));
		run;

*** Calculation of the results ***;

		data est;
			  merge ytot xtot;
		      &by by  &classes;
  			  array ybs{&b};
			  array xbs{&b};
			  array rbs{&b};
			  length var1 $ 8;
			  length var2 $ 8;
			  length type $ 8;
			  if xhat ne 0 then Estimate=((yhat/xhat));
			  rep_mod=0;
			  do i=1 to &b;
			       if xbs{i} ne 0 then rbs{i}=((ybs{i}/xbs{i}));
				   *** Calculation of the number of non-empty replicates ***;
				   if xbs{i} ne 0 and xbs{i} ne . then rep_mod=rep_mod+1;  
			  end;
			  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of rbs1-rbs&b)))/rep_mod;
			  bs_sd=sqrt(bs_var);
			  if Estimate ne 0 then bs_cv=abs(round((bs_sd/Estimate)*100,.01));
			  else bs_cv=.;
			  &CI_low=Estimate-&level*bs_sd;
			  &CI_upp=Estimate+&level*bs_sd;
			  var1="&var1";
			  var2="&var2";
			  type="Ratio";
			  if rep_mod ne &b then call symput('flag_rat',1);
			  drop ybs1-ybs&b xbs1-xbs&b rbs1-rbs&b xhat yhat i _type_ _freq_;
		run;

*** Calculation of the design effect (if activated at the beginning of the program) ***;

		%if (&deff=1) %then %do;
			proc means data=bs_data noprint vardef=wdf;
			  var &var1 &var2;
			  weight &fwgt;
			&by  by &classes;
			  output out=deff1 n=nsamp mean=ybar xbar var=yvar xvar;

			proc corr data=bs_data cov vardef=wdf outp=deff2 noprint;
			  var &var1 &var2;
			  weight &fwgt;
			&by  by &classes;

			data deff2;
			  set deff2;
			  where _type_='COV' and upcase(_name_)=left(upcase("&var1"));
			  rename &var2=cov;
			  drop _name_ &var1 _type_;

			&by proc sort data=deff1;
			&by 	by &classes;
			&by run;
			&by proc sort data=deff2;
			&by 	by &classes;
			&by run;

			data est;
			  merge est deff1 deff2 (in=in1);
			&by  by &classes;
		 	  if in1;
			  
			  if xbar not in (0,.) and ybar not in (0,.) then do;
			     vsrs=(1/nsamp)*(ybar**2)/(xbar**2)*
			          (yvar/ybar**2+xvar/xbar**2-2*cov/xbar/ybar);
			     deff=bs_var/vsrs;
			  end;
			 drop nsamp ybar xbar yvar xvar cov vsrs _type_ _freq_; 

			proc datasets library=work;
			  delete deff1 deff2;
			run;
		%end;

*** End of the design effect calculation ***;

*** Creation of the result file ***;

		&by   data est;
		&by      set est;
		&by  			%do k=1 %to 10;
		&by  			cla&k='        ';
		&by  			%end;
		&by  			%do k=1 %to &number; 
		&by  			cla&k=input(compress(&&cla&k),$8.);
		&by  			%end;
		&by   run;

		data allrats;
			set allrats est;
		run;

		%let printrat=1;

		proc datasets library=work;
		    delete ytot xtot ytot_bsw xtot_bsw est n;
		run;

%mend ratio;



************************************************
****                                        ****
****      DIFFERENCE BETWEEN RATIOS         ****
****                                        ****
************************************************;


%macro diff_rat(var1,var2,var3,var4);

*** Determine if negative values are present ***;

		%let negative=0;
		data _NULL_;
			set bs_data (keep=&var1 &var2 &var3 &var4);
			if &var1 ne 0 and &var1 / abs(&var1)=-1  then do; call symput('negative',1); end;
			if &var2 ne 0 and &var2 / abs(&var2)=-1  then do; call symput('negative',1); end;
			if &var3 ne 0 and &var3 / abs(&var3)=-1  then do; call symput('negative',1); end;
			if &var4 ne 0 and &var4 / abs(&var4)=-1  then do; call symput('negative',1); end;
		run;

*** If no negative values are present, use the fast method ***;

	%if (&negative=0) %then %do;
		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var1;
		  &by  class &classes;
		  output out=ytot
		  sum=yhat ybs1-ybs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var2;
		  &by  class &classes;
		  output out=xtot
		  sum=xhat xbs1-xbs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var3;
		  &by  class &classes;
		  output out=yytot
		  sum=yyhat yybs1-yybs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var4;
		  &by  class &classes;
		  output out=xxtot
		  sum=xxhat xxbs1-xxbs&b;
		run;
	%end;

*** If negative values are present, use the long method ***;

	%if (&negative=1) %then %do;

		OPTIONS nonotes;

		proc means data=bs_data noprint;
		     var &var1;
		     weight &fwgt;
		&by  class &classes;
		     output out=ytot sum=yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var1;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=ytot_bsw sum=ybs&looper;
			run;

			&by proc sort data=ytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot ytot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		     var &var2;
		     weight &fwgt;
		&by  class &classes;
		  output out=xtot sum=xhat;
		run;

		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var2;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=xtot_bsw sum=xbs&looper;
			run;

			&by proc sort data=xtot_bsw;
			&by 	by &classes;
			&by	run;
			
			data xtot;	
				merge xtot xtot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		     var &var3;
		     weight &fwgt;
		&by  class &classes;
		  output out=yytot sum=yyhat;
		run;

		&by proc sort data=yytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var3;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=yytot_bsw sum=yybs&looper;
			run;

			&by proc sort data=yytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data yytot;	
				merge yytot yytot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		     var &var4;
		     weight &fwgt;
		&by  class &classes;
		  output out=xxtot sum=xxhat;
		run;

		&by proc sort data=xxtot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var4;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=xxtot_bsw sum=xxbs&looper;
			run;

			&by proc sort data=xxtot_bsw;
			&by 	by &classes;
			&by	run;
			
			data xxtot;	
				merge xxtot xxtot_bsw;
				&by by &classes;
			run;
		%end;
		OPTIONS notes;
	%end;

*** Calculation of the sample size  ***;

		proc means data=Mfile noprint;
		     var &var1;
		     where &var1>0;
		&by  class &classes;
		  output out=n1 n=n1;
		run;

		proc means data=Mfile noprint;
		     var &var3;
		     where &var3>0;
		&by  class &classes;
		     output out=n3 n=n3;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;
		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;
		&by proc sort data=yytot;
		&by 	by &classes;
		&by run;
		&by proc sort data=xxtot;
		&by 	by &classes;
		&by run;
		&by proc sort data=n1;
		&by 	by &classes;
		&by run;
		&by proc sort data=n3;
		&by 	by &classes;
		&by run;

		data ytot;
			merge ytot n1 (drop=_type_ _freq_) n3 (drop=_type_ _freq_);
		&by by &classes;
			if (&negative=1) then do; n1=.; n3=.; end;  * Set the sample size to missing if negative values are present to avoid wrong interpretations;
		run;

*** Variables for the confidence level ;

		%let level=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('CI' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('CI_low',compress("CI_l&CI"));
	 		call symput ('CI_upp',compress("CI_u&CI"));
		run;

*** Calculation of the results ***;

	  data est;
		  merge ytot xtot yytot xxtot;
		  &by 	by &classes;
		  array ybs{&b};
		  array xbs{&b};
		  array yybs{&b};
		  array xxbs{&b};
		  array drbs{&b};
		  length var1 $ 8;
		  length var2 $ 8;
		  length var3 $ 8;
		  length var4 $ 8;
		  length type $ 10;
		  if xhat ne 0 and xxhat ne 0 then Estimate=(((yhat/xhat)-(yyhat/xxhat)));
		  rep_mod=0;
		  do i=1 to &b;
		    if xbs{i} ne 0 and xxbs{i} ne 0 then drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i})));
			*** Calculation of the number of non-empty replicates ***;
		    if xbs{i} ne 0 and xbs{i} ne . and xxbs{i} ne 0 and xxbs{i} ne . then rep_mod=rep_mod+1;  
		  end;
		  if rep_mod ne 0 then bs_var=&r*(((rep_mod-1)*(var(of drbs1-drbs&b)))/rep_mod);
		  bs_sd=sqrt(bs_var);
		  if Estimate ne 0 then bs_cv=abs(round((bs_sd/Estimate)*100,.01));
		  else bs_cv=.;
		  if bs_sd ne 0 then z=Estimate/bs_sd;
		  pvalue=2*(1-probnorm(abs(z)));
		  &CI_low=Estimate-&level*bs_sd;
		  &CI_upp=Estimate+&level*bs_sd;
		  var1="&var1";
		  var2="&var2";
		  var3="&var3";
		  var4="&var4";
		  type="Dif_Rat";
		  if rep_mod ne &b then call symput('flag_dif',1);
		  drop ybs1-ybs&b xbs1-xbs&b yybs1-yybs&b xxbs1-xxbs&b drbs1-drbs&b
		       xhat yhat xxhat yyhat i _type_ _freq_;

		  &by  			%do k=1 %to 10;
		  &by  			cla&k='        ';
		  &by  			%end;
		  &by  			%do k=1 %to &number; 
		  &by  			cla&k=input(compress(&&cla&k),$8.);
		  &by  			%end;
		  run;


*** Creation of the result file ***;

	  	data diffrat;
			set diffrat est;
	    run;

		%let printdif=1;

		proc datasets library=work;
		    delete ytot xtot yytot xxtot est n1 n3;
		run;


%mend diff_rat;



************************************************
****                                        ****
****             PERCENTILE                 ****
****                                        ****
************************************************;


%macro prcntle(var,perc);

*** Calculation of the percentiles ***;

		proc univariate data=bs_data noprint;
		     var &var;
		     weight &fwgt;
		&by  by &classes;
		     output out=ytot pctlpts=&perc pctlpre=_ pctlname=P&perc._yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		options nonotes;

		%do looper=1 %to &B;

			proc univariate data=bs_data noprint;
		  		var &var;
		  		weight bsw&looper;
			&by  by &classes;
		    output out=pctl pctlpts=&perc pctlpre=_ pctlname=P&perc._ybs&looper;
			run;

			&by proc sort data=pctl;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot(in=a) pctl(in=b);
				&by by &classes;
				if a and b;
				label _P&perc._yhat='yhat';
			run;

		%end;

		options notes;

*** Calculation of the sample size  ***;

		proc means data=Mfile noprint;
		    var &var;
		    where &var>0;
		&by  class &classes;
		    output out=n n=n;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		&by proc sort data=n;
		&by 	by &classes;
		&by run;

		data ytot;
			merge ytot (in=a) n (drop=_type_ _freq_);
		&by by &classes;
			if a;
		run;

*** Identification of the empty replicates (ybs are recoded to missing)***;

		proc means data=bs_data noprint;
		    var bsw1-bsw&b;
		&by  by &classes;
		    output out=ytot_nul
		         sum= ybs_nul1-ybs_nul&b;
		run;

		data ytot (drop=ybs_nul1-ybs_nul&b);
			merge ytot ytot_nul;
		&by  by &classes;
			array _P&perc._ybs{&b};
			array ybs_nul{&b};
			do i = 1 to &b;
				if ybs_nul{i}=0 or ybs_nul{i}=. then _P&perc._ybs{i}=.;
			end;
		run;

*** Variables for the confidence level ;

		%let level=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('CI' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('CI_low',compress("CI_l&CI"));
	 		call symput ('CI_upp',compress("CI_u&CI"));
		run;

*** Calculation of the results ***;

	   data est;
		  set ytot(rename=(_P&perc._yhat=yhat _P&perc._ybs1-_P&perc._ybs&b=ybs1-ybs&b));
	  	  array ybs{&b};
		  length var $ 8;
		  length perc $ 8;
		  length type $ 8;
		  Estimate=yhat;
		    *** Calculation of the number of non-empty replicates ***;
		  	rep_mod=0;
		    do i=1 to &b;
		  	if ybs{i} ne . then rep_mod=rep_mod+1;  
		    end;
		  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of ybs1-ybs&b)))/rep_mod;
		  bs_sd=sqrt(bs_var);
		  if Estimate ne 0 then bs_cv=round((bs_sd/Estimate)*100,.01);
		  else bs_cv=.;
		  &CI_low=Estimate-&level*bs_sd;
		  &CI_upp=Estimate+&level*bs_sd;
		  var="&var";
		  perc="&perc";
		  type="Prcntle";
		  if rep_mod ne &b then call symput('flag_perc',1);
		  drop ybs1-ybs&b yhat i ;
	   run;

*** Creation of the result file ***;

		%let printperc=1;

		data allperc;
		   set allperc est;
		run;

		proc datasets library=work;
		   delete ytot_nul ytot pctl est n;
		run;


%mend prcntle;



************************************************
****                                        ****
****          LINEAR REGRESSION             ****
****                                        ****
************************************************;


%macro regress(yvar,xvar);

		%let model_reg=&model_reg+1;
		OPTIONS nonotes;

*** Point estimation ***;

		proc reg data=bs_data outest=orig(keep=&classes &inter &xvar) noprint;
		  model &yvar=&xvar;
		  weight &fwgt;
		  &by  by &classes;
		run;

		proc transpose data=orig out=origest(drop=_label_) prefix=beta name=param;
		  var &inter &xvar;
		  &by  by &classes;
		run;

*** Calculation of the bootstrap variance ***;

		data _NULL_; 
		    L=int((&b/10)+0.999);
		    call symput ('L' , trim(left(L))); 
		run;

		%let j_dep=1;

		%do k=1 %to 10;
		  %let j=%eval(1+((&k-1)*&L));
		  %let kL=%eval(&k*&L);

		data _NULL_;
		if (&b - &kL) >0  then do;   
		                         k=&k;
		                         j=1+((&k-1)*&L);
		                         kL=&k*&L;
		                       end;
		                  else do; 
		                         k=10; 
		                         j=&j_dep;
		                         kL=&b;
		                       end;

		call symput ('k' , trim(left(k)));
		call symput ('j' , trim(left(j)));
		call symput ('kL' , trim(left(kL)));
		run;

		data weight (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
		  set bs_data;
		run;

		  %do i=&j %to &kL;

		   %put  &i linear regressions completed;

		   %let j_dep=%eval(&kl+1);

	ods listing close;

		proc reg data=weight;
		     model &yvar=&xvar;
		     weight bsw&i;
		&by  by &classes;
		ods output parameterestimates=regout;
		   run;

		proc summary data=regout;
		&by  by &classes;
				var df;
				output out = regoutdf sum = sumdf;
			run;

		data regout;
			set regout;
			ind=1;
		run;
		data regoutdf;
			set regoutdf;
			ind=1;
		run;

		data regout2;
			merge regoutdf regout;
		    by &classes ind; 
			if sumdf < _freq_ or sumdf=. then estimate=.;
			best&i=estimate;
		run;

		   %if (&i =1) %then %do;

		     data bsbeta;
		       set regout2 (keep=&classes best&i variable);
		     run;

		   %end;
		   %else %do;

			 data bsbeta;
			    merge bsbeta regout2 (keep=&classes best&i variable);
			 &by    by &classes;
			 run;

		   %end;
	
		ods listing;

		  %end;
		%end;

		OPTIONS notes;

*** Identification of the empty replicates ("best" are recoded to missing)***;

		proc means data=bs_data noprint;
		    var bsw1-bsw&b;
		&by  by &classes;
		    output out=bsbeta_nul
		         sum= ybs_nul1-ybs_nul&b;
		run;

		data bsbeta (drop=ybs_nul1-ybs_nul&b);
			merge bsbeta bsbeta_nul;
		&by  by &classes;
			array best{&b};
			array ybs_nul{&b};
			do i = 1 to &b;
				if ybs_nul{i}=0 then best{i}=.;
			end;
		run;

*** Variables for the confidence level ;

		%let level=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('CI' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('CI_low',compress("CI_l&CI"));
	 		call symput ('CI_upp',compress("CI_u&CI"));
		run;

*** Calculation of the results ***;

		data est;
		  merge origest bsbeta;
		  &by 	by &classes;
		  rename beta1=Beta;
			*** Calculation of the number of non-empty replicates ***;
			rep_mod=0;
		  	array best{&b};
			do i=1 to &b;
			if best{i} ne . then rep_mod=rep_mod+1; 
		    end; 
		  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of best1-best&b)))/rep_mod;
		  bs_sd=sqrt(bs_var);
		  if bs_sd ne 0 then t=beta1/bs_sd;
		  pvalue=2*(1-probnorm(abs(t)));  * The normal distribution is used as an approximation of the t distribution;
		  dep="&yvar";
		  mod=&model_reg;
		  Model=compress(mod)||': Dependent Variable = '|| compress(dep) ; 
		  drop best1-best&b dep _type_ _freq_ i;
		  if rep_mod ne &b then call symput('flag_reg',1);
		run;

		data bs_reg;
		    set bs_reg est;
		run;

*** Creation of the result file ***;

		%let printreg=1;

		proc datasets library=work;
		    delete regout regout2 regoutdf bsbeta bsbeta_nul origest est orig weight;
		run;


%mend regress;



************************************************
****                                        ****
****         LOGISTIC REGRESSION            ****
****                                        ****
************************************************;


%macro logreg(yvar,xvar);

		%let model_log=&model_log+1;
		OPTIONS nonotes;

*** Point estimation ***;

		proc logistic data=bs_data outest=orig(keep=&classes &inter &xvar) descending noprint;
		  model &yvar=&xvar;
		  &by by &classes;
		  weight &fwgt;
		run;

		proc transpose data=orig out=origest prefix=beta name=param;
		  var &inter &xvar;
		  &by by &classes;
		run;

*** Calculation of the bootstrap variance ***;

		data _NULL_; 
		    L=int((&b/10)+0.999);
		    call symput ('L' , trim(left(L))); 
		run;

		%let j_dep=1;

		%do k=1 %to 10;
		  %let j=%eval(1+((&k-1)*&L));
		  %let kL=%eval(&k*&L);

		data _NULL_;
		if (&b - &kL) >0  then do; 
		                          k=&k;
		                          j=1+((&k-1)*&L);
		                          kL=&k*&L;
		                       end;
		                  else do; 
		                          k=10; 
		                          j=&j_dep;
		                          kL=&b;
		                       end;

		call symput ('k' , trim(left(k)));
		call symput ('j' , trim(left(j)));
		call symput ('kL' , trim(left(kL)));
		run;


		data weight (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
		  set bs_data;
		run;

		  %do i=&j %to &kL;

		   %put  &i logistic regressions completed;
		 
		    %let j_dep=%eval(&kl+1);

	ods listing close;

			proc logistic data=weight descending;
		      model &yvar=&xvar;
		      &by by &classes;
		      weight bsw&i;
			ods output parameterestimates=regout;
		    run;

		proc summary data=regout;
		&by  by &classes;
				var df;
				output out = regoutdf sum = sumdf;
			run;

		data regout;
			set regout;
			ind=1;
		run;
		data regoutdf;
			set regoutdf;
			ind=1;
		run;

		data regout2;
			merge regoutdf regout;
		    by &classes ind; 
			if sumdf < _freq_ or sumdf=. then estimate=.;
			best&i=estimate;
		run;

		    %if (&i =1) %then %do;

		     data bsbeta;
		       set regout2 (keep=&classes best&i variable);
		     run;

		   %end;
		   %else %do;

			 data bsbeta;
			    merge bsbeta regout2 (keep=&classes best&i variable);
			 &by    by &classes;
			 run;

		   %end;
	
		ods listing;

		  %end;
		%end;

		OPTIONS notes;

*** Identification of the empty replicates ("best" are recoded to missing)***;

		proc means data=bs_data noprint;
		    var bsw1-bsw&b;
		&by  by &classes;
		    output out=bsbeta_nul
		         sum= ybs_nul1-ybs_nul&b;
		run;

		data bsbeta (drop=ybs_nul1-ybs_nul&b);
			merge bsbeta bsbeta_nul;
		&by  by &classes;
			array best{&b};
			array ybs_nul{&b};
			do i = 1 to &b;
				if ybs_nul{i}=0 then best{i}=.;
			end;
		run;

*** Variables for the confidence level ;

		%let level=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('CI' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('CI_low',compress("CI_l&CI"));
	 		call symput ('CI_upp',compress("CI_u&CI"));
		run;

*** Calculation of the results ***;

		data est;
		  merge origest bsbeta;
		  &by 	by &classes;
		  rename beta1=Beta;
		    *** Calculation of the number of non-empty replicates ***;
			rep_mod=0;
		  	array best{&b};
			do i=1 to &b;
			if best{i} ne . then rep_mod=rep_mod+1; 
			end; 
		  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of best1-best&b)))/rep_mod;
		  bs_sd=sqrt(bs_var);
		  if bs_sd ne 0 then Wald=(beta1/bs_sd)*(beta1/bs_sd);
		  pvalue=1-probchi(Wald,1);
		  odds=exp(beta1);
		  lo=beta1-&level*bs_sd;
		  hi=beta1+&level*bs_sd;
		  &CI_low=exp(lo);
		  &CI_upp=exp(hi);
		  dep="&yvar";
		  mod=&model_log;
		  Model=compress(mod)||': Dependent Variable = '|| compress(dep) ; 
		  drop best1-best&b dep _type_ _freq_ i;
		  if rep_mod ne &b then call symput('flag_log',1);
		run;

		data bs_reglg; 
		     set bs_reglg est;
		run;

*** Creation of the result file ***;

		%let printlog=1;

		proc datasets library=work;
		    delete regout regout2 regoutdf bsbeta bsbeta_nul origest est orig weight;
		run;


%mend logreg;



************************************************
****                                        ****
****                 CHI-2                  ****
****                                        ****
************************************************;

**** Note: the calculation method is the same as the one used by Wesvar;


*** This is only to allow users to have formats with the data set used ***;

		proc format;
			value toblankexisting;
		run;


%macro chi2(var1,var2);

%let printchi2=1;
%let test_chi2=&test_chi2+1;

	%local nvar class_list temp;

	%if "&classes" ne " " %then %do;
		%let temp = %sysfunc(compbl(&classes));
		*** Insert a comma between the variables in the CLASSES list, for use in SQL procedure  ***;
		%let class_list = %sysfunc(translate(&temp, ",", " "));

		***	Tokenize classes	***;
		%let nvar = 1;
		%let v = %scan(&classes,&nvar);
		%do %while (%length(&v) > 0);
			data _NULL_;
				call symput('clas'!!left(&nvar),"&v");
			run;

			%let nvar = %eval(&nvar+1);
			%let v = %scan(&classes,&nvar);
		%end; /* while */
		%let nvar = %eval(&nvar-1);
	%end; /* if classes */

**	Derive normalized weights to derive proper PEARSON test	***;

    proc sql;
		create table nwgt as
		select %if "&classes" ne " " %then &class_list,; 
				&var1, &var2, &fwgt/mean(&fwgt) as normwgt
		from bs_data
		%if "&classes" ne " " %then
		group by &class_list;
		;
	quit;

***	Deriving estimated table using normalized final weights	***;

	proc freq data = nwgt noprint;
	&by by &classes;
		table &var1 * &var2/chisq outpct out = temp0;
		weight normwgt;
		output out = chi2test pchi;
	run;

***	Read results to issue warning in the presence of low cells table ***;

	proc sql;
		create table chi2test as
		select distinct c.*, case 
						when t.count <= 5 then '(*)'
						else '   ' end as warn_msg
		from chi2test as c, temp0 as t
		%if "&classes" ne " " %then %do;
		where t.percent ne . 
			%do i = 1 %to &nvar;
				and c.&&clas&i = t.&&clas&i
			%end;
		%end;
		order by %if "&classes" ne " " %then 
					%do i = 1 %to &nvar;
						c.&&clas&i,
					%end;
				 warn_msg desc
		;
	quit;

	data chi2test;
		set chi2test;
		by &classes;
		%if "&classes" ne " " %then %do;
			if first.&&clas&nvar;	
		%end;
		%else %do;
			if _N_ = 1;
		%end;
	run;


%local lowcell;
%let lowcell = 0;	/* Initialize flag to trigger in the presence of low cells */

***	Toggle flag value to indicate lowcell table(s)	***;
	data temp0;
		set temp0;
		if percent;
		/* convert percentages into decimal numbers */
		percent = percent / 100;
		if count le 5 then
			call symput('lowcell',1);
	run;

***	Deriving replicate tables using replicate weights	***;

	%local _rep;

	%do _rep = 1 %to &b;
	*** Subsetting only chunks of 50 bsw at a time for more efficiency ***;
	%if %eval((&_rep - 1)/50) = %sysevalf((&_rep-1)/50,ceil) %then %do;
		data __;
			set bs_data (keep = &classes &var1 &var2 bsw&_rep-bsw%sysfunc(min(&_rep+49,&b)));
		run;
	%end;
		proc freq data = __ noprint;
	&by 		by &classes;
			table &var1 * &var2/outpct out = sortie;
			weight bsw&_rep;
		run;

		data sortie;
			set sortie;
			if percent;
			rep = &_rep;
			percent = percent/100;
		run;

**	appending results in REP_TABLES	***;

		%if &_rep = 1 %then %do;
			data rep_tables;
				set sortie;
			run;
		%end;
		%else %do;
			proc append data = sortie base = rep_tables;
			run;
		%end;
%end;
	
proc sql noprint;
	drop table sortie
	;
quit;

***	Count number of observations, store in a macro variable	***;

	%local nobs dsid rc;
		%let dsid = %sysfunc(open(bs_data));
	%let nobs = %sysfunc(attrn(&dsid,nobs));
		%let rc = %sysfunc(close(&dsid));
 
***	Pick up the number of observations by CLASSES variables	  ***;

    %if "&classes" ne " " %then %do;
		proc summary data = bs_data;
			class &classes;
			output out = classification (where = (&clas1
				%do i = 2 %to &nvar;
					and &&clas&i
				%end;	));
		run;
	%end;

***	Calculate marginal percentages	***;

	proc summary data = temp0;
		class &classes &var1 &var2;
		var percent;
		output out = marg sum = margin;
	run;

	proc sql;
		create table marg0 as
		select distinct t.*, a.margin as pr, b.margin as pc
		from temp0 as t, marg as a, marg as b
		where /* extracting row marginals from MARG as A */
				(t.&var1 = a.&var1 and  a.&var2 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = a.&&clas&i
					%end;)
		and /* extracting column marginals from MARG as B */
				(t.&var2 = b.&var2 and  b.&var1 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = b.&&clas&i
					%end;)
		;
	quit;

*** 	... and do the same for all replicates	***;

	proc summary data = rep_tables;
		class rep &classes &var1 &var2;
		var percent;
		output out = marg sum = margin;
	run;

	proc sql;
		create table marginales as
		select distinct t.*, a.margin as pr, b.margin as pc
		from rep_tables as t, marg as a, marg as b
		where (t.&var1 = a.&var1 and t.rep = a.rep and  a.&var2 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = a.&&clas&i
					%end;)
			  and (t.&var2 = b.&var2 and t.rep = b.rep and  b.&var1 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = b.&&clas&i
					%end;)
		order by rep, %if "&classes" ne " " %then &class_list,; 
				 &var1, &var2
		;
	quit;

***	Deriving variance estimates (i.e. Vij, Vi. and V.j				***
***							or	   VARPIJ, VC and VR respectively)	***;

proc sql;
	*************
	*	CELLS   *
	*************;
		create table vij0 as
		select distinct r.rep, 
						%if "&classes" ne " " %then
							%do i = 1 %to &nvar;
								r.&&clas&i,
							%end;
						r.&var1, r.&var2, t.percent as p0, 
			 (r.percent - t.percent)**2  as diffs
		from marginales as r, marg0 as t
		where t.&var1 = r.&var1 and t.&var2 = r.&var2
				%if "&classes" ne " " %then
				  	%do i = 1 %to &nvar;
						and t.&&clas&i = r.&&clas&i 
					%end;
		;
		create table vij as
		select distinct %if "&classes" ne " " %then &class_list,; &var1, &var2, p0,
		/* Use approach where empty reps are ignored and only non-empty reps are used	*/
			   &r*(1/count(*))*sum(diffs) as varpij
		from vij0
		group by %if "&classes" ne " " %then &class_list,; &var1, &var2
		;

	*************
	*	ROWS    *
	*************;
		create table vi0 as
		select distinct rep, 
				%if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						r.&&clas&i,
					%end;
			 r.&var1, 
			 t.pr as pr0,
			 (r.pr - t.pr)**2  as diffs_r
		from marginales as r, marg0 as t
		where t.&var1 = r.&var1 
				%if "&classes" ne " " %then
				  	%do i = 1 %to &nvar;
						and t.&&clas&i = r.&&clas&i 
					%end;
		;
		create table vi as
		select distinct %if "&classes" ne " " %then &class_list,; &var1, pr0,
		/* Use approach where empty reps are ignored and only non-empty reps are used	*/
			   &r*(1/count(*))*sum(diffs_r) as vr
		from vi0
		group by %if "&classes" ne " " %then &class_list,; &var1
		;

	***************
	*	COLUMNS	  *
	***************;
		create table vj0 as
		select distinct rep, 
				%if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						r.&&clas&i,
					%end;
			 r.&var2, 
			 t.pc as pc0,
			 (r.pc - t.pc)**2  as diffs_c
		from marginales as r, marg0 as t
		where t.&var2 = r.&var2
				%if "&classes" ne " " %then
				  	%do i = 1 %to &nvar;
						and t.&&clas&i = r.&&clas&i 
					%end;
		;
		create table vj as
		select distinct %if "&classes" ne " " %then &class_list,; &var2, pc0,
		/* Use approach where empty reps are ignored and only non-empty reps are used	*/
			   &r*(1/count(*))*sum(diffs_c) as vc
		from vj0
		group by %if "&classes" ne " " %then &class_list,; &var2
		;

*** Gather all variances (cells and marginals) together in the table VARPIJ ***;

	create table varpij as 
	select distinct %if "&classes" ne " " %then
						%do i = 1 %to &nvar;
							r.&&clas&i,
						%end;
		   r.&var1, r.&var2, p0, pr0, pc0,
		   varpij, vr, vc
	from vi as i, vj as j, vij as r
	where i.&var1 = r.&var1 and j.&var2 = r.&var2
		  %if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and i.&&clas&i = r.&&clas&i 
				and j.&&clas&i = r.&&clas&i 
				and i.&&clas&i = j.&&clas&i
			%end;
	;

**************************************************
*	   RS3 (Rao-Scott Second order adjusted)     *
**************************************************;

***	Preparing input for RS3 ***;

	*** Calculate ratios Vi./Pi. for upcoming sum  ***;
		create table vipi as
		select distinct %if "&classes" ne " " %then &class_list,; &var1, vr/pr0 as ratr
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var1
		;

	*** Calculate ratios V.j/P.j for upcoming sum  ***;
		create table vjpj as
		select distinct %if "&classes" ne " " %then &class_list,; &var2, vc/pc0 as ratc
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var2
		;
		quit;

***	For some reason, it happens there are duplicate rows, the next 2 steps get rid of those...	***;

	data vipi;
		set vipi;
		by &classes &var1;
		if first.&var1;
	run;

	data vjpj;
		set vjpj;
		by &classes &var2;
		if first.&var2;
	run;

	proc sql;

	*** Sum on rows of Vi./Pi. ***;
		create table vipi2 as
		select distinct %if "&classes" ne " " %then &class_list,; 
				sum(ratr) as sum_vi_pi
		from vipi
		%if "&classes" ne " " %then group by &class_list;
		;

	*** Sum on columns of V.j/P.j ***;
		create table vjpj2 as
		select distinct %if "&classes" ne " " %then &class_list,; 
				sum(ratc) as sum_vj_pj
		from vjpj
		%if "&classes" ne " " %then group by &class_list;
		;

	*** Sum on rows and columns of Vij/(Pi.*P.j) ***;
		create table vijpipj as
		select distinct %if "&classes" ne " " %then &class_list,; 
				sum(varpij/(pr0*pc0)) as sum_vij_pipj
		from varpij
		%if "&classes" ne " " %then group by &class_list;
		;

	*** Put all the previous sums together ***;
		create table vps as
		select v.*, i.sum_vi_pi, j.sum_vj_pj, ij.sum_vij_pipj
		from varpij as v, vipi2 as i, vjpj2 as j, vijpipj as ij
		%if "&classes" ne " " %then %do;
		where v.&clas1 = i.&clas1 and v.&clas1 = j.&clas1 and v.&clas1 = ij.&clas1 
			%do i = 2 %to &nvar;
				and v.&&clas&i = i.&&clas&i and v.&&clas&i = j.&&clas&i and v.&&clas&i = ij.&&clas&i 
			%end;
		%end;
		;
	quit;


***	A lookup table of all combinations of Replicate number, &CLASSES, VAR1 and	***
***	VAR2 is required to handle cases where some replicates have empty cells 	***
***	which are otherwise represented in the original cross tabulation (i.e. with ***
***	the final weights). REPS contains an instance of all replicate numbers, 	***
***	which will be used to make a cartesian product with the cross tabulation of	***
***	&CLASSES, VAR1 and VAR2.													***;

	data reps;
		do rep = 1 to &b;
			output;
		end;
	run;

***	The number of observation by &CLASSES (_FREQ_) is needed to compute the 	***
***	vector Y, where all elements are multiplied by SQRT(n). [n <- _FREQ_]		***;

	%if "&classes" ne " " %then %do;
		proc sql;
			create table vps as
			select v.*, _FREQ_
			from vps as v, classification as c
			where v.&clas1 = c.&clas1
				  %do i = 2 %to &nvar;
				  	and v.&&clas&i = c.&&clas&i
				  %end;
			;
			create table rep_tables as
			select r.*, f._FREQ_
			from rep_tables as r, classification as f
			where r.&clas1 = f.&clas1
				  %do i = 2 %to &nvar;
				  	and r.&&clas&i = f.&&clas&i
				  %end;
			;

	***	Create lookup table mentionned above	***;

		create table lut as
		select distinct rep, 
						%do i = 1 %to &nvar;
							a.&&clas&i,
						%end; 
						&var1, &var2, _freq_
		from reps, 
			 (select distinct &class_list, &var1 from marg0) as a,
			 (select distinct &class_list, &var2 from marg0) as b,
			 classification as f
		where a.&clas1 = f.&clas1 and b.&clas1 = f.&clas1 and a.&clas1 = b.&clas1
			  %do i = 2 %to &nvar;
			  	and a.&&clas&i = f.&&clas&i and b.&&clas&i = f.&&clas&i and a.&&clas&i = b.&&clas&i
			  %end;
		;
	%end;
	%else %do;
		data vps;
			set vps;
			_FREQ_ = &nobs;
		run;
		data rep_tables;
			set rep_tables;
			_FREQ_ = &nobs;
		run;

		proc sql;

	***	Create lookup table mentionned above in the case where there is no &CLASSES	***;

			create table lut as
			select distinct rep, &var1, &var2, &nobs as _FREQ_
			from reps, marg0
			;
	%end;

***	REP_TABLES might have some rows not included. This happens when some 	***
***	replicates have empty cells which are otherwise represented in the 		***
***	original cross tabulation. Left joining the lookup table created above	***
***	by REP_TABLES will generate lines where PERCENT is missing. We update	***
***	them to zeros afterwards.												***;

	create table rep_tables2 as
	select l.*, count, percent
	from lut as l left join rep_tables as a 
	on l.rep = a.rep and
	   l.&var1 = a.&var1 and l.&var2 = a.&var2
	   %if "&classes" ne " " %then 
		%do i = 1 %to &nvar;
			and l.&&clas&i = a.&&clas&i
		%end;
	order by l.rep, 
		  %if "&classes" ne " " %then
	   		%do i = 1 %to &nvar;
				&&clas&i,
			%end; 
			l.&var1, l.&var2
	;
	update rep_tables2
	set percent = 0, count = 0
	where percent = .
	;

	***	Compute the Ys for estimates (YS) and for replicates (YS_REP)	***;

	create table lut_r as
	select distinct %if "&classes" ne " " %then 
						%do i = 1 %to &nvar;
							l.&&clas&i,
						%end;
		   l.&var1, pr0, l._freq_
	from lut as l left join vps as v
	on l.&var1 = v.&var1
		%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and l.&&clas&i = v.&&clas&i 
			%end;
	;
	create table lut_c as
	select distinct %if "&classes" ne " " %then 
						%do i = 1 %to &nvar;
							l.&&clas&i,
						%end;
		   l.&var2, pc0
	from lut as l left join vps as v
	on l.&var2 = v.&var2
		%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and l.&&clas&i = v.&&clas&i 
			%end;

	;
	create table lut_rc as
	select %if "&classes" ne " " %then
			 %do i = 1 %to &nvar;
				r.&&clas&i,
			 %end;
		   r.&var1, c.&var2, pr0, pc0, _freq_
	from lut_c as c, lut_r as r
	%if "&classes" ne " " %then %do;
	where r.&clas1 = c.&clas1
			%do i = 1 %to &nvar;
				and r.&&clas&i = c.&&clas&i
			%end;
	%end;
	;
	create table ys as
	select %if "&classes" ne " " %then
			 %do i = 1 %to &nvar;
				l.&&clas&i,
			 %end;
		   l.&var1, l.&var2, p0, l.pr0, l.pc0, l._freq_
	from lut_rc as l left join vps as v
	on l.&var1 = v.&var1 and l.&var2 = v.&var2 
		%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and l.&&clas&i = v.&&clas&i
			%end;
	order by l.&var1, l.&var2
	;
	quit;

	data ys;
		set ys;
		if p0 = . then
			p0 = 0;
		y = sqrt(_FREQ_)*(p0 - (pr0*pc0));
		p = pr0*pc0;
	run;

	proc sql;
		create table ys_rep as
		select *, sum(percent) as pr
		from rep_tables2
		/*	Doing it by VAR1 to have ROW proportions	*/
		group by rep, %if "&classes" ne " " %then &class_list,; &var1
		;
		create table ys_rep as
		select *, sum(percent) as pc
		from ys_rep
		/*	Doing it by VAR2 to have COLUMN proportions	*/
		group by rep, %if "&classes" ne " " %then &class_list,; &var2
		order by rep, %if "&classes" ne " " %then &class_list,; &var1, &var2
		;
		create table ys_rep as
		select *, sqrt(_FREQ_)*(percent - (pr*pc)) as yg
		from ys_rep
		order by rep, %if "&classes" ne " " %then &class_list,; &var1, &var2
		;
		/* Generate the (Yig-Yi) for each replicate	*/
		create table yig_yi as
		select r.*, t.p, yg - y as yig_yi
		from ys_rep as r left join ys as t
		on r.&var1 = t.&var1 and r.&var2 = t.&var2 
			%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and r.&&clas&i = t.&&clas&i 
			%end;
		order by r.rep, %if "&classes" ne " " %then &class_list,; r.&var1, r.&var2
		;
	quit;

	data yig_yi;
		set yig_yi;
		by rep &classes &var1 &var2;
		%if "&classes" ne " " %then %do;
			if first.&&clas&nvar then i = 0;
		%end;
		%else %do;
			if first.rep then i = 0;
		%end;
		/* Create an index for the next step */
		i + 1;
	run;


	proc sql;
	***	Read the table YIG_YI twice, once as the (Yig-Yi) and the second time as	***
	***	the (Yjg-Yj) to derive the Bij. 											***;
		create table bij as
		select distinct %if "&classes" ne " " %then 
							%do i = 1 %to &nvar;
								i.&&clas&i,  
							%end;
				i.i as i, 
				j.i as j, 
				i.p as pi, 
				j.p as pj, 
				(1/&b)*sum(i.yig_yi * j.yig_yi) as Bij
		from yig_yi as i, yig_yi as j
		where i.rep = j.rep 
			%if "&classes" ne " " %then 
				%do i = 1 %to &nvar;
					and i.&&clas&i = j.&&clas&i 
				%end;
		group by %if "&classes" ne " " %then 
				%do i = 1 %to &nvar;
					i.&&clas&i,  
				%end;
				i.&var1, i.&var2, j.&var1, j.&var2
		;

***	Create RS3 results ***;

	%if "&classes" ne " " %then %do;
		create table rs3 as
		select distinct %do i = 1 %to &nvar;
							a.&&clas&i,
						%end;
				c.warn_msg, 
				b.bii_pi,	/* Numerator in the DF (nu) formula (not squared yet) */
				c._PCHI_, 
				sum((Bij**2)/(pi*pj)) as BijPiPj	/* Denominator in the DF (nu) formula */
		from bij as a, 
				(/* Sub-query to extract Bii/Pi, i.e. Bij's where i = j */
				 select distinct &class_list, sum(Bij/pi) as bii_pi 
				 from bij 
				 where i=j 
				 group by &class_list
				) as b,
			 chi2test as c
		where a.&clas1 = b.&clas1 and a.&clas1 = c.&clas1
			  %do i = 2 %to &nvar;
				and a.&&clas&i = b.&&clas&i and a.&&clas&i = c.&&clas&i
			  %end;
		group by a.&clas1
				 %do i = 2 %to &nvar;
					, a.&&clas&i
				 %end;
		;
	%end;
	%else %do;
		create table rs3 as
		select distinct c.warn_msg, 
			   b.bii_pi, 	/* Numerator in the DF (nu) formula (not squared yet) */
			   c._PCHI_, 
			   sum((Bij**2)/(pi*pj)) as BijPiPj	/* Denominator in the DF (nu) formula */
		from bij as a,
			(/* Sub-query to extract Bii/Pi, i.e. Bij's where i = j */
			 select distinct sum(Bij/pi) as bii_pi 
			 from bij 
			 where i=j
			) as b,
			 chi2test as c
		;
	%end;
	quit;


****************************************************
*	 Prepare for printing of the CHI-2 results	   *
****************************************************;

	proc sql;
		create table chi2 as
		select distinct %if "&classes" ne " " %then &class_list,;
			   "&var1" as var1, "&var2" as var2,
			   "R.-S. sec. order adj. chi-2 "!!warn_msg as statistic, 
			   (bii_pi**2)/bijpipj as Deg_Freedom,
			   ((bii_pi**2)/bijpipj)*_PCHI_/bii_pi as chi2_value,
			   1 - probchi((bii_pi**2)/bijpipj*_PCHI_/bii_pi, (bii_pi**2)/bijpipj) as p_value
		from rs3
		;
	quit;

	data chi2;
		set chi2;
		test_num=&test_chi2;
		Test = compress(test_num)||':   Variables:  '|| compress(var1) ||'  VS  '||compress(var2); 
	run;

	data allchi2;
		set allchi2 chi2;
	run;


***	Create a cross-tabulation containing required stats	***;

	proc sql noprint;
		select type
		into :type1
		from sashelp.vcolumn
		where libname = "WORK" and memname = "VARPIJ" and upcase(name) = "%upcase(&var1)"
		;
		select type
		into :type2
		from sashelp.vcolumn
		where libname = "WORK" and memname = "VARPIJ" and upcase(name) = "%upcase(&var2)"
		;
	quit;

	proc sql;
		create table cross_in as
		select %if "&classes" ne " " %then &class_list,;
			   &var1, &var2, 
			   p0 label = "Percent",
			   varpij label = "Variance",
			   sqrt(varpij)/p0 as cv label = "_C.V._"
		from varpij
		order by %if "&classes" ne " " %then &class_list,;&var1, &var2
		;

		create table cross_row as
		select distinct %if "&classes" ne " " %then &class_list,; &var1,
			   pr0 label = "Percent",
			   vr label = "Variance",
			   sqrt(vr)/pr0 as cv label = "_C.V._"
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var1
		;

		create table cross_col as
		select distinct %if "&classes" ne " " %then &class_list,;
			   %if &type1 = num %then .; %else " "; as &var1, &var2, 
			   pc0 label = "Percent",
			   vc label = "Variance",
			   sqrt(vc)/pc0 as cv label = "_C.V._"
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var1, &var2
		;
	quit;

	proc transpose data = cross_row out=row_trans(drop=_name_ rename = (col1 = total));
		by &classes &var1;
		var pr0 vr cv;
	run;

	proc transpose data=cross_col out = col_trans(drop=_name_);
		by &classes &var1;
		id &var2;
		format &var2;
	run;

	proc transpose data = cross_in out = in_trans (drop = _name_);
		by &classes &var1;
		id &var2;
		format &var2;
	run;

	%if &type2 = num %then %do;
		proc sql noprint;
			select count(distinct &var2)
			into :ncolumns
			from temp0
			;
		quit;
		data _null_;
			call symput('colx',compress('val'!!&ncolumns));
		run;
		proc sql noprint;
			select distinct &var2 format=toblankexisting.
			into :val1-:&colx
			from temp0
			;
		quit;

		data col_trans;
			set col_trans;
			%do i = 1 %to &ncolumns;
				%let valeur = _&&val&i;
				rename &valeur = &var2&valeur;
				format &valeur 11.4;
			%end;
		run;

		data in_trans;
			set in_trans;
			%do i = 1 %to &ncolumns;
				%let valeur = _&&val&i;
				rename &valeur = &var2&valeur;
				format &valeur 11.4;
			%end;
		run;
	%end;

	data col_trans;
		set col_trans;
		total=.;
		var1="&var1";
		var2="&var2";
	run;

	data temp;
		var1="&var1";
		var2="&var2";
	run;

	proc sql;
		create table crosstab as
		select a.*, b.total, c.*
		from in_trans as a, row_trans as b, temp as c
		where %if "&classes" ne " " %then 
				%do i = 1 %to &nvar;
					a.&&clas&i = b.&&clas&i and
				%end;		
			  a.&var1 = b.&var1 and
			  a._label_ = b._label_
		 union 
		select *
		from col_trans
		;
	quit;

	data crosstab (drop =  test_num var1 var2) ;
		set crosstab;
		test_num=&test_chi2;
		call symput ('test_num2' ,compress(&test_chi2));
		Test = compress(test_num)||':   Variables:  '|| compress(var1) ||'  VS  '||compress(var2); 
	run;

	data crosstab&test_num2;
		set crosstab;
	run;

	proc datasets library=work;
	      delete chi2 ;
	run;
	quit;
 
	%if "&classes" ne " " %then %do;
	proc sql;
			drop table bij, chi2test, classification, lut, lut_r, lut_c, lut_rc, marg, marg0, 
				   marginales, nwgt, reps, rep_tables, rep_tables2, rs3, temp0,  
				   varpij, vijpipj, vipi, vipi2, 
				   vjpj, vjpj2, vi, vi0, vij, vij0, vj, vj0, vps, yig_yi,ys, ys_rep, __,
                   crosstab, cross_col, cross_in, cross_row, in_trans, row_trans, col_trans, temp;
	quit;
	%end;
	%else %do;
	proc sql;
			drop table bij, chi2test, lut, lut_r, lut_c, lut_rc, marg, marg0, 
				   marginales, nwgt, reps, rep_tables, rep_tables2, rs3, temp0,  
				   varpij, vijpipj, vipi, vipi2, 
				   vjpj, vjpj2, vi, vi0, vij, vij0, vj, vj0, vps, yig_yi,ys, ys_rep, __,
                   crosstab, cross_col, cross_in, cross_row, in_trans, row_trans, col_trans, temp;
	quit;
	%end;

	title;

%mend chi2;  *** END OF THE CHI-2 MACRO ***;



*********************************************************************************
*** Section 5: DECLARATION OF THE MACROS FOR PRINTING OF THE RESULTS          ***
***       See the documentation for a description of the results              ***
*********************************************************************************;


****************;

%macro prnttot;

****************;

%if &printtot=1 %then %do;

*** Preparation of the result file when breakdown variable(s) are present ***;

		&by  data alltots;
		&by  	set alltots (drop = &classes);
		&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
		&by     %do i=1 %to &number;
		&by  	     if cla&i=" " then cla&i="All";
		&by  	     if cla&i="All" then ind&i=1;  &by  else ind&i=2;
		&by     &&cla&i=cla&i;
		&by  	%end;
		&by  	run;

		&by   proc sort data=alltots;
		&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
		&by   run;

*** Printing of the results of the macro TOTAL ***;

		proc print data=alltots label noobs;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Variance Estimation for a TOTAL";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
		   %if (&deff=1) %then %do;
		   var &classes var n Estimate bs_sd bs_cv &CI_low &CI_upp deff;
		   format Estimate bs_sd &CI_low &CI_upp deff 11.2;
		   label  Estimate = "Total"
				  bs_sd = "Standard error"
				  bs_cv = "Coeff. of variation"
				  var = "Variable"
				  n = "Sample size" 
				  &CI_low = "Lower limit confidence interval &CI%"
				  &CI_upp = "Upper limit confidence interval &CI%"
				  deff = "Design effect";
		   %end;
		   %else %do;
		   var &classes var n Estimate bs_sd bs_cv &CI_low &CI_upp;
		   format Estimate bs_sd &CI_low &CI_upp 11.2;
		   label  Estimate = "Total"
				  bs_sd = "Standard error"
				  bs_cv = "Coeff. of variation"
				  var = "Variable"
				  n = "Sample size" 
				  &CI_low = "Lower limit confidence interval &CI%"
				  &CI_upp = "Upper limit confidence interval &CI%";
		   %end;
		run;
%end;

%mend prnttot;


****************;

%macro prntrat;

****************;

%if &printrat=1 %then %do;

*** Preparation of the result file when breakdown variable(s) are present ***;

		&by  data allrats;
		&by  	set allrats(drop = &classes);
		&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
		&by     %do i=1 %to &number;
		&by  	     if cla&i=" " then cla&i="All";
		&by  	     if cla&i="All" then ind&i=1;  &by  else ind&i=2;
		&by     &&cla&i=cla&i;
		&by  	%end;
		&by  	run;

		&by   proc sort data=allrats;
		&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
		&by   run;

*** Printing of the results of the macro RATIO ***;

		proc print data=allrats label noobs;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Variance Estimation for a RATIO";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
		   label  Estimate = "Ratio"
				  bs_sd = "Standard error"
				  bs_cv = "Coeff. of variation"
				  var1 = "Numerator"
				  var2 = "Denominator"
				  n1 = "Numerator size" 
				  &CI_low = "Lower limit confidence interval &CI%"
  				  &CI_upp = "Upper limit confidence interval &CI%"
				  rep_mod = "Number of replicates used";
		   %if (&deff=1 and &flag_rat=0) %then %do;
			 var &classes var1 var2 n1 Estimate bs_sd bs_cv &CI_low &CI_upp deff;
		     format  bs_sd &CI_low &CI_upp Estimate 11.4 deff 11.2;
		     label deff = "Design effect";
		   %end;
		   %else %if (&deff=0 and &flag_rat=0) %then %do;
			 var &classes var1 var2 n1 Estimate bs_sd bs_cv &CI_low &CI_upp;
		     format  bs_sd &CI_low &CI_upp Estimate 11.4;
		   %end;
		   %if (&deff=1 and &flag_rat=1) %then %do;
			 var &classes var1 var2 n1 Estimate bs_sd bs_cv &CI_low &CI_upp deff rep_mod;
		     format  bs_sd &CI_low &CI_upp Estimate 11.4 deff 11.2;
		     label deff = "Design effect";
			 	footnote1 "-----------------------------------------------------------------------------------------------------";
				footnote2 "  Warning: Some bootstrap replicates were not used for variance calculation.  These replicates ";
                footnote3 "           did not contain any observations contributing to the denominator.";
				footnote4 "           Refer to the column 'Number of replicates used' for the number used.";
				footnote5 "-----------------------------------------------------------------------------------------------------";
		   %end;
		   %else %if (&deff=0 and &flag_rat=1) %then %do;
			 var &classes var1 var2 n1 Estimate bs_sd bs_cv &CI_low &CI_upp rep_mod;
		     format  bs_sd &CI_low &CI_upp Estimate 11.4;
			 	footnote1 "-----------------------------------------------------------------------------------------------------";
				footnote2 "  Warning: Some bootstrap replicates were not used for variance calculation.  These replicates ";
                footnote3 "           did not contain any observations contributing to the denominator.";
				footnote4 "           Refer to the column 'Number of replicates used' for the number used.";
				footnote5 "-----------------------------------------------------------------------------------------------------";
		   %end;
		run;
%end;
footnote;

%mend prntrat;

****************;

%macro prntdiff;

****************;

%if &printdif=1 %then %do;

*** Preparation of the result file when breakdown variable(s) are present ***;

		&by  data diffrat;
		&by  	set diffrat(drop = &classes);
		&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
		&by     %do i=1 %to &number;
		&by  	     if cla&i=" " then cla&i="All";
		&by  	     if cla&i="All" then ind&i=1;  &by  else ind&i=2;
		&by     &&cla&i=cla&i;
		&by  	%end;
		&by  	run;

		&by   proc sort data=diffrat;
		&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
		&by   run;


*** Printing of the results of the macro DIFF_RAT ***;

	   proc print data=diffrat label noobs width=minimum;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Variance Estimation for a DIFFERENCE BETWEEN RATIOS";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
		   label bs_sd = "Std. err."
                 Estimate = "Difference of ratios"
		   		 bs_cv = "C.V."
				 var1 = "Num1"
				 var2 = "Den1"
				 var3 = "Num2"
				 var4 = "Den2"
				 n1 = "Num1 size" 
				 n3 = "Num2 size" 
				 pvalue = "p value"
				 &CI_low = "Lower limit confidence interval &CI%"
				 &CI_upp = "Upper limit confidence interval &CI%"
				 rep_mod = "Number of replicates used";
		   %if (&flag_dif=0) %then %do;
	       var &classes var1 var2 var3 var4 n1 n3 Estimate z pvalue bs_sd bs_cv &CI_low &CI_upp;
	       format  bs_sd &CI_low &CI_upp Estimate pvalue 11.4 z 11.2;
		   %end;
		   %else %do;
	       var &classes var1 var2 var3 var4 n1 n3 Estimate z pvalue bs_sd bs_cv &CI_low &CI_upp rep_mod;
	       format  bs_sd &CI_low &CI_upp Estimate pvalue 11.4 z 11.2;
			 	footnote1 "-----------------------------------------------------------------------------------------------------";
				footnote2 "  Warning: Some bootstrap replicates were not used for variance calculation.  These replicates ";
                footnote3 "           did not contain any observations contributing to one or both denominator(s).";
				footnote4 "           Refer to the column 'Number of replicates used' for the number used.";
				footnote5 "-----------------------------------------------------------------------------------------------------";
           %end;
		   run;
%end;
footnote;

%mend prntdiff;


****************;

%macro prntperc;

****************;

%if &printperc=1 %then %do;

		&by   proc sort data=allperc;
		&by     by &classes;
		&by   run;

*** Printing of the results of the macro PRCNTLE ***;

		proc print data=allperc label noobs;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Variance Estimation for a PERCENTILE";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
		   label bs_sd = "Standard error"
				 bs_cv = "Coeff. of variation"
				 var = "Variable"
				 perc = "Percentile (1-99)"
				 Estimate = "Percentile value"
				 n = "Sample size" 
				 &CI_low = "Lower limit confidence interval &CI%"
				 &CI_upp = "Upper limit confidence interval &CI%"
				 rep_mod = "Number of replicates used";
		   %if (&flag_perc=0) %then %do;
		   var &classes var perc n Estimate bs_sd bs_cv &CI_low &CI_upp;
		   format Estimate bs_sd bs_cv &CI_low &CI_upp 11.2;
		   %end;
		   %else %do;
		   var &classes var perc n Estimate bs_sd bs_cv &CI_low &CI_upp rep_mod;
		   format Estimate bs_sd bs_cv &CI_low &CI_upp 11.2;
			 	footnote1 "------------------------------------------------------------------------------------------------------------------";
				footnote2 "  Warning: Some bootstrap replicates were not used for variance calculation.  These replicates ";
                footnote3 "           did not contain any observations having the caracteristic of intereset in the estimation ";
				footnote4 "           domain (breakdown variables). Refer to the column 'Number of replicates used' for the number used.";
				footnote5 "------------------------------------------------------------------------------------------------------------------";
           %end;
		run;
%end;
footnote;

%mend prntperc;

****************;

%macro prntreg;

****************;

%if &printreg=1 %then %do;

*** Printing of the results of the macro REGRESS ***;

		proc print data=bs_reg label noobs;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Variance Estimation for a LINEAR REGRESSION";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
		   label bs_sd = "Standard error"
				 pvalue = "p value"
				 param = "Independent variables"
				 rep_mod = "Number of replicates used";
		   by Model;
		   %if (&flag_reg=0) %then %do;
	       var &classes param beta bs_sd t pvalue ;
		   format t  11.2 beta bs_sd pvalue 11.4;
		   %end;
		   %else %do;
	       var &classes param beta bs_sd t pvalue rep_mod;
		   format t  11.2 beta bs_sd pvalue 11.4;
			 	footnote1 "-------------------------------------------------------------------------------------------------------------------";
				footnote2 "  Warning: Some bootstrap replicates were not used for variance calculation. This can happen because these  ";
                footnote3 "           replicates did not contain any observations in the estimation domain (breakdown variables).   ";
                footnote4 "           Another possibility is that the model is not full rank and least-squares solutions for the       ";
				footnote5 "           parameters are not unique. Refer to the column 'Number of replicates used' for the number used.";
				footnote6 "-------------------------------------------------------------------------------------------------------------------";
		   %end;
	  	run;
%end;
footnote;

%mend prntreg;

****************;

%macro prntlog;

****************;

%if &printlog=1 %then %do;

*** Printing of the results of the macro LOGREG ***;

		proc print data=bs_reglg label noobs;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Variance Estimation for a LOGISTIC REGRESSION";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
		   label odds = "Odds ratio"
		   		 bs_sd = "Standard error"
				 pvalue = "p value"
				 &CI_low = "Odds ratio lower limit conf. int. &CI%"
				 &CI_upp = "Odds ratio upper limit conf. int. &CI%"
				 param = "Independent variables"
				 rep_mod = "Number of replicates used";
		   by Model;
		   %if (&flag_log=0) %then %do;
	       var  &classes param beta odds bs_sd Wald pvalue &CI_low &CI_upp;
		   format odds Wald 11.2 beta bs_sd pvalue &CI_low &CI_upp 11.4;
		   %end;
		   %else %do;
	       var  &classes param beta odds bs_sd Wald pvalue &CI_low &CI_upp rep_mod;
		   format odds Wald 11.2 beta bs_sd pvalue &CI_low &CI_upp 11.4;
			 	footnote1 "-------------------------------------------------------------------------------------------------------------------";
				footnote2 "  Warning: Some bootstrap replicates were not used for variance calculation. This can happen because these  ";
                footnote3 "           replicates did not contain any observations in the estimation domain (breakdown variables).   ";
                footnote4 "           Another possibility is that the model is not full rank and least-squares solutions for the       ";
				footnote5 "           parameters are not unique. Refer to the column 'Number of replicates used' for the number used.";
				footnote6 "-------------------------------------------------------------------------------------------------------------------";
		   %end;
		run;
%end;
footnote;

%mend prntlog;

****************;

%macro prntchi2;

****************;

%if &printchi2=1 %then %do;

*** Printing of the results of the macro CHI2 ***;

	 	   proc print data=allchi2 noobs label;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Rao-Scott Second order adjusted CHI-SQUARE test of indepedence";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
	       var  &classes statistic deg_freedom chi2_value p_value ;
		   format chi2_value 11.2 p_value 11.4;
		   label deg_freedom = "Deg. of freedom"
		   		 chi2_value = "Chi-square value"
				 p_value = "p value"
				 statistic = "Statistic";
		  by test;
				footnote1 "-----------------------------------------------------------------------------------------------";
				footnote2 "  Note: An asterisk (*) in the column Statistic indicates that the table has cell(s) with less  ";
				footnote3 "        than 5 unweighted observations.  Chi-square may not be a valid test in that case.";
				footnote4 "-----------------------------------------------------------------------------------------------";
		  run;

		footnote;

		%do i=1 %to &test_num2;
		   proc print data=crosstab&i noobs label;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "CHI-SQUARE test of indepedence:  Various statistics for the two-way table";
	       title5 "using &B bootstrap replicates ";
		   title6 ;
		   label _label_ = "Statistic";
		   by test;
		   format total 11.4;
	 	   run;
		%end;
%end;

%mend prntchi2;

****************;

%macro output;

****************;

*** Execution of the macros for printing ***;

	%prnttot;
	%prntrat;
	%prntdiff;
	%prntperc;
	%prntreg;
	%prntlog;
	%prntchi2;

	proc datasets library=work;
	      delete Mfile bsamp version tmp;
	run;
	quit;

%mend output;



/* End of MACROE_V31.SAS program */
